function [FID] = solver_LRD(FID,c,mask,signal,lamda)
disp('Low rank Hankel decoupling algorithm starts...')
%% Solver for the low rank Hankel decoupling method
% Author : Tianyu Qiu
% Email  : tianyuqiu@stu.xmu.edu.cn
% Feb. 18, 2023.
% References: 
% [1] Tianyu Qiu, Amir Jahangiri, Xiao Han, Dmitry Lesovoy, Tatiana Agback, Peter Agback, Adnane Achour, Xiaobo Qu*, Vladislav Orekhov*. Resolution enhancement of NMR by decoupling with low-rank Hankel model, arXiv preprint arXiv:2212.01144, 2022.
% [2] Xiaobo Qu*, Maxim Mayzel, Jian-Feng Cai, Zhong Chen, Vladislav Orekhov*. Accelerated NMR spectroscopy with low-rank reconstruction, Angewandte Chemie International Edition, 54(3):852-854, 2015.
% Note: This function used in [1] is modified from low rank Hankel solver written in [2] by incorporating J-coupling information.
%% Input: 
% FID -> (NUS) J-coupled FID
% c -> J-coupling function
% mask -> Sampling schedule
% signal -> Full sampled J-coupled FID
% lamda -> Regularization parameter
%% Output:
% FID -> Reconstructed decoupled FID
%%
    MaxIter_Inner=2e3;
    beta_const=2^0;
    tao=1;
    Tol=1e-5;
    PeakRatio=0.50;
%%
NormFactor=max(abs(fft2c(signal(:))));
FID=FID/NormFactor; 
%%
han_num=round(PeakRatio*numel(FID));
RFx=vec2hankel(FID,han_num);
OverLap=hankel2vec(ones(size(RFx)),han_num);
%%
D=ones(size(RFx));
Z=zeros(size(RFx));
%%
lamdaUChy=lamda*c.*mask.*FID;
lamdaUChUC=lamda*c.*c.*mask;
%%
Diff_x_inner=1e10;
iter_inner=1;
FID_last_inner=FID;
%%
    if (tao>(1.618*beta_const))
        error('tao is larger than 1.618*beta! Set a smaller tao.')
    end
    thr=1/beta_const;
    left_inv=(lamdaUChUC+beta_const*OverLap).^(-1);
    while (iter_inner<MaxIter_Inner)&&(Diff_x_inner>Tol)
        %% Normal equations
        D_over_beta=D/beta_const;
        right=lamdaUChy+beta_const*(hankel2vec(Z-D_over_beta,han_num));
        FID=left_inv.*right;
        %% Singular thresholding
        RFx=vec2hankel(FID,han_num);
        [U, S, V]=svd(RFx+D_over_beta,'econ');
       %%
        S=diag(S);
        S=max(S-thr,0);
        r = sum(S(:)>0);
        Z = U(:,1:r)*diag(S(1:r))*V(:,1:r)';
        %% Update D
        D=D+tao*(RFx-Z);
        %% Update tolerance
        diff_x_inner=FID-FID_last_inner;
        Diff_x_inner=norm(diff_x_inner(:))/norm(FID_last_inner(:));
        FID_last_inner=FID;
        iter_inner=iter_inner+1;       
    %%
    end
    FID = FID*NormFactor;
end
